# What is Axiom?

Axiom is a tool for standardizing the technology decisions of your organization, in a decoupled, modular way.  This lets you focus on the development of features, while using standards for file location, project deploys, asset pipelines, etc.

This is a role typically assumed by Frameworks.  The goal of Axiom though is to be more of a 'framework toolkit', where you still have the freedom to swap out technologies, and leave out things you don't want.

Features of Axiom include:

* workflow standardization
* an extensible application kernel

The Axiom infrastructure is separated into four layers:

* Runtime
* Protocol
* Plugins

In addition, your application can be separated into:

* Application Configs
* Application Code

# Runtime

The Runtime is responsible for directing control flow.  It interprets the Protocol as a configuration.  Based on the Protocol, it calls Plugins/Services at the appropriate locations, at times when it deems appropriate.

Because the Plugins are constrained to a Protocol, and not to a specific Runtime implementation, it is our hope that the Runtime layer can continue to evolve without disrupting the Plugin ecosystem.

# Protocol

The Protocol layer contains a group of Interfaces, each of which describes the lifecycles of workflows and runtimes for the application.  The Interfaces are JSON-like configuration files - they may be in memory data structures with require statements or other inderections, but the resulting data structure should be pure data and should not contain any functions.

Protocols must be given a name and version, and a definitive set of Interfaces.  This allows Plugins and Runtime modules to identify themselves as being compatible with a given Protocol.  The Protocol system allows for many 'Frameworks' to exist within the Axiom ecosystem, ideally leading to a more consistent experience for both application developers and framework developers.

# Plugins

The Plugins bring technologies from the Node.js ecosystem into the Axiom ecosystem.  They are the bridge between general technologies and how those technologies should plug into specific workflows and runtimes.  A given Plugin can extend many different Interfaces - for instance, installing the Mongoose Plugin you should expect that it 

# Design Decisions

Within Axiom, we have selected Decoupled Architecture in place of Generated Boilerplate.  This is not an exclusive decision.  Our preference towards an architectural approach comes from the belief that this will lead to more inclusive systems which embrace and make use of the myriad of technologies currently available.

# Implementation Decisions

The core of Axiom's interworkings are implemented as communications on a message bus - Postal.js.
